#!/usr/bin/env python
# coding=UTF-8
# 
# Copyright (C) 2010 Stefanos Harhalakis
#
# This file is part of wifieye.
#
# wifieye is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# wifieye is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with wifieye.  If not, see <http://www.gnu.org/licenses/>.
#
# $Id: 0.py 2265 2010-02-21 19:16:26Z v13 $

__version__ = "$Id: 0.py 2265 2010-02-21 19:16:26Z v13 $"

import gtk
import gobject
import hildon
from hildondesktop import *
from gtk import gdk
from math import pi
import cairo
import time

from portrait import FremantleRotation
import launcher
from xdg.IconTheme import getIconPath


import config
import apps


class Icons:
    def __init__(self):
	self.mode='l'


    def load(self):
	x=0
	y=0
	fn=["maegirls", "wifieye"]*3
	for f in fn:
	    dt=apps.readOne(f)
	    ico=getIconPath(dt['icon'], config.iconsize)
	    print "ico:", ico
	    p=gtk.gdk.pixbuf_new_from_file_at_size(ico, config.iconsize,
		config.iconsize)
	    print x, y
	    #dt={'icon2': p}
	    dt['icon2']=p
	    print x, y, dt
	    self.icons[x][y].setApp(dt)
	    x+=1
	    if x>=config.size:
		x=0
		y+=1
#	    self.icons.append(p)

	print "end of Icons init"

    def setMode(self, mode):
	self.mode=mode

class Icon:
    def __init__(self):
	self.icon=None
	self.lastpress=0
	self.ispressed=False

	self.x=0
	self.y=0

	self.presstime=0.25

	self.window=None

	self.clickcount=0

    def timePressed(self):
	""" return how much time a button is pressed """
	dt=time.time() - self.lastpress

	return(dt)

    def setApp(self, dt):
	self.name=dt['id']
	self.icon=dt['icon2']

    def draw(self, cr, x, y, mode):
	self.x=x
	self.y=y

	if self.icon==None:
	    return

	cr.save()
	cr.set_source_rgba(0.1, 0.1, 0.1, 1)
	cr.set_line_width(5)

	if self.ispressed:
	    t=1.0 * min(self.timePressed(), self.presstime) / self.presstime
	    g=0.3+0.5*t
	    b=0.3+0.7*t
	    cr.set_source_rgba(0, g, b, 0.7)
	else:
	    cr.set_source_rgba(0.3, 0.3, 0.3, 0.7)
	x3=x + (config.iconspace/6)
	y3=y + (config.iconspace/6)

	r=10	# Radius
	w=config.iconsize+(config.iconspace*2/3)

	cr.move_to(x3+r, y3)
	cr.arc(x3+w-r,  y3+r,   r,	    pi*1.5, pi*2)
	cr.arc(x3+w-r,  y3+w-r, r,	    0,	    pi*0.5)
	cr.arc(x3+r,    y3+w-r, r,	    pi*0.5, pi)
	cr.arc(x3+r,    y3+r,   r,	    pi,	    pi*1.5)

	cr.stroke_preserve()
	cr.fill()
#	    cr.paint()
	cr.restore()

	icon=self.icon

	if mode=='l':
	    icon2=icon
	else:
	    icon2=icon.rotate_simple(gdk.PIXBUF_ROTATE_COUNTERCLOCKWISE)

	cr.save()
	x3=x + (config.iconspace/2)
	y3=y + (config.iconspace/2)
	cr.set_source_pixbuf(icon2, x3, y3)
	cr.paint()
	cr.restore()

    def timerPressed(self):
	#print "timer"
	if not self.ispressed:
	    return(False)

	self.invalidate()

	if self.timePressed()>self.presstime:
	    ret=False
	else:
	    ret=True
	return(ret)

    def doPress(self):
	# Double-time: time for pressed and time for not-pressed
	if time.time() - self.lastpress > self.presstime*2:
	    self.clickcount=0

	self.lastpress=time.time()
	self.ispressed=True
	gobject.timeout_add(20, self.timerPressed)

    def doLaunch(self):
	print "launch:", self.name
	launcher.launch(self.name)

    def doTrippleClick(self):
	print "tripple"
	aps=apps.scan()
	
	lst=[aps[x]['name'] for x in aps]
	lst.sort()

	# TODO: EDO EDO EDO
	# Na doylevei o selector, na dialegei efarmogi kai na
	# efarmozetai sto sygkekrimeno eikonidio
	#
	# Na brethei lysh kai gia ta kena
	dialog=gtk.Dialog('App select', None,
	    gtk.DIALOG_DESTROY_WITH_PARENT, buttons=())

	selector=hildon.TouchSelectorEntry(text=True)
	selector.set_column_selection_mode(
	    hildon.TOUCH_SELECTOR_SELECTION_MODE_SINGLE)
	dialog.vbox.pack_start(selector, True, True, 0)
	dialog.set_size_request(0,900)

	for app in lst:
	    if app==None:
		continue
	    selector.append_text(app)

	dialog.show_all()

	r=dialog.run()
	print "r:", r
#	prog=hildon.Program.get_instance()

	print lst

    def doRelease(self):
	dt=time.time() - self.lastpress
	self.ispressed=False
	if dt>self.presstime and dt<2:
	    self.doLaunch()
	elif dt<self.presstime:
	    self.clickcount+=1
	    if self.clickcount==3:
		self.doTrippleClick()
		self.clickcount=0
    
    def doCancel(self):
	self.ispressed=False

    def invalidate(self, window=None):
	if window==None:
	    window=self.window
	else:
	    self.window=window
	w=config.iconsize + config.iconspace
	rect=gdk.Rectangle(self.x, self.y, w, w)
	gdk.Window.invalidate_rect(window, rect, True)

class DrlaunchPlugin(HomePluginItem, Icons, FremantleRotation):
    def __init__(self):
	HomePluginItem.__init__(self)
	Icons.__init__(self)
	FremantleRotation.__init__(self, 'DrlaunchPlugin')

	launcher.init()

	self.setMode('l')

	w=config.size * config.iconsize + (config.size) * config.iconspace
	self.set_size_request(w, w)

	sz=config.size
	icons=[]
	for x in xrange(sz):
	    icons.append([])
	    for y in xrange(config.size):
		icons[x].append(Icon())

	self.icons=icons

	self.lasticon=None  # The last icon that got selected

	self.load()

    def iconAt(self, x, y):
	""" Get icon at coordinates x,y. X and Y are in pixels """

	w=config.iconsize + config.iconspace

	if self.mode=='l':
	    x2=int(x / w)
	    y2=int(y / w)
	else:
	    x2=config.size - int(y/w) - 1
	    y2=int(x/w)

	#print x2, y2
	ret=self.icons[x2][y2]

	return(ret)

    def _draw(self, cr):
	x=0
	y=0

#	print "mode:", self.mode
#	print "icons", len(self.icons)
	for l in self.icons:
	    for i in l:
#		print x, y

		if self.mode=='l':
		    x2=x * (config.iconsize + config.iconspace)
		    y2=y * (config.iconsize + config.iconspace)
		else:
		    x2=y * (config.iconsize + config.iconspace)
		    y2=(config.size-x-1) * (config.iconsize + config.iconspace)

#		print "draw:", x, y
		ico=self.icons[x][y]
		ico.draw(cr, x2, y2, self.mode)

		x+=1
		if x>=config.size:
		    x=0
		    y+=1

    def do_expose_event(self, event):
	#print "do_expose"

	cr=self.window.cairo_create()

	cr.rectangle(event.area.x, event.area.y,
	    event.area.width, event.area.height)

	cr.clip()

	self._draw(cr)

#	HomePluginItem.do_expose_event(self, event)

    def setLastIcon(self, icon):
	if icon==self.lasticon:
	    return

	if self.lasticon!=None:
	    self.lasticon.doCancel()
	    self.lasticon.invalidate(self.window)
	self.lasticon=icon

    def do_realize(self):
	screen=self.get_screen()
	self.set_colormap(screen.get_rgba_colormap())
	self.set_app_paintable(True)

	HomePluginItem.do_realize(self)

    def do_button_press_event(self, event):
	#print "press", event.type
	icon=self.iconAt(event.x, event.y)
#	rect=gdk.Rectangle(event.x,event.y,1,1)
#	rect=gdk.Rectangle(0, 0, 100, 100)
	icon.doPress()
	icon.invalidate(self.window)
	self.setLastIcon(icon)

#	gdk.Window.invalidate_rect(self.window, rect, True)

	return(True)

    def do_button_release_event(self, event):
	#print "release"
	icon=self.iconAt(event.x, event.y)
	icon.doRelease()
	icon.invalidate(self.window)
	self.setLastIcon(None)
	return(True)

    def do_leave_notify_event(self, event):
	#print "leave", event.x, event.y
	self.setLastIcon(None)
	return(True)

    def do_pproperty_notify_event(self, event):
	#print "property"
	icon=self.iconAt(event.x, event.y)
	icon.doCancel()
	icon.invalidate(self.window)
	return(True)

    def do_motion_notify_event(self, event):
	#print "motion"
	self.setLastIcon(None)
#	icon=self.iconAt(event.x, event.y)
#	icon.doCancel()
#	icon.invalidate(self.window)
	return(True)

    def do_button_press_event_old(self, event):
	#print "press"
	if event.type==gdk.BUTTON_PRESS:
	    print "press", event.type
	    if self.mode=='p':
		self.setMode('l')
	    else:
		self.setMode('p')
	    self.queue_draw()
	return True

    # For debugging
    def do_event1(self, event):
	print "event:", event, event.type

    def butTest(self, arg):
	print "but", arg

    def on_orientation_changed(self, orientation):
	print "orch:", orientation
	o=orientation[0]
	self.setMode(o)
	self.queue_draw()

hd_plugin_type = DrlaunchPlugin

def do1():
#    gobject.type_register(MyQ)
    gobject.type_register(hd_plugin_type)
    obj=gobject.new(hd_plugin_type, plugin_id="plugin_id")
    obj.show_all()
    gtk.main()

def do2():
    win=DrlaunchPlugin()
    win.connect('delete-event', gtk.main_quit)

    print "win:", win

#    t=DrlaunchPlugin()
#    win.add(t)

    win.show_all()
    gtk.main()

if __name__=="__main__":
    do1()



# vim: set ts=8 sts=4 sw=4 noet formatoptions=r ai nocindent:

